<?php

if (!defined('_PS_VERSION_')) {
    exit;
}

class Ucc extends Module
{
    /**
     * Ucc constructor.
     */
    public function __construct()
    {
        $this->name = 'ucc';
        $this->tab = 'analytics_stats';
        $this->version = '1.0.1';
        $this->author = 'Deviniti';
        $this->need_instance = 1;
        $this->bootstrap = true;
        $this->ps_versions_compliancy = [
            'min' => '1.7.1.0',
            'max' => '1.8',
        ];

        parent::__construct();

        $this->displayName = $this->l('UCC Prestashop');
        $this->description = $this->l('UCC integration module');
        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?');
    }

    /**
     * @return bool
     */
    public function install()
    {
        return (parent::install()
            && $this->registerHook('displayOrderConfirmation')
            && $this->registerHook('displayProductAdditionalInfo')
            && $this->registerHook('actionFrontControllerSetMedia')
            && Configuration::updateValue('UCC_PLUGIN_TOKEN', '')
        );
    }

    /**
     * @return bool
     */
    public function uninstall()
    {
        Configuration::deleteByName('UCC_PLUGIN_TOKEN');

        return parent::uninstall();
    }

    /**
     * @return bool
     */
    private function isConfigured()
    {
        if (!empty(strval(Configuration::get('UCC_PLUGIN_TOKEN')))) {
            return true;
        }

        return false;
    }

    /**
     * @return bool
     */
    private function validatePluginToken()
    {
        $apiUrl = 'https://api.ucancommerce.com/api/v1/plugin/installation';
        $data = json_encode([
            'pluginToken' => strval(Tools::getValue('UCC_PLUGIN_TOKEN')),
            'shopPlatform' => 'PRESTASHOP',
            'platformVersion' => _PS_VERSION_
        ]);
        $headers = array(
            "Content-Type: application/json",
        );

        $curl = curl_init($apiUrl);
        curl_setopt($curl, CURLOPT_URL, $apiUrl);
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_POSTFIELDS, $data);

        curl_exec($curl);

        $code = curl_getinfo($curl, CURLINFO_HTTP_CODE);
        curl_close($curl);

        if ($code == 200) {
            return true;
        }

        return false;
    }

    /**
     * @return string
     */
    public function getContent()
    {
        $output = '<h2>UCC</h2>';

        if (Tools::isSubmit('submit'.$this->name)) {
            $pluginToken = Tools::getValue('UCC_PLUGIN_TOKEN');

            if (!$pluginToken || empty($pluginToken) || !$this->validatePluginToken()) {
                $output .= $this->displayError($this->l('Invalid Configuration value'));
            } else {
                Configuration::updateValue('UCC_PLUGIN_TOKEN', $pluginToken);
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }

        return $output.$this->displayForm();
    }

    /**
     * @return mixed
     */
    public function displayForm()
    {
        // Get default language
        $defaultLang = (int)Configuration::get('PS_LANG_DEFAULT');

        // Init Fields form array
        $fieldsForm[0]['form'] = [
            'legend' => [
                'title' => $this->l('Settings'),
                'icon'  => 'icon-cogs'
            ],
            'input' => [
                [
                    'type' => 'text',
                    'label' => $this->l('Plugin token'),
                    'name' => 'UCC_PLUGIN_TOKEN',
                    'size' => 20,
                    'min' => 36,
                    'max' => 36,
                    'required' => true
                ]
            ],
            'submit' => [
                'title' => $this->l('Save'),
                'class' => 'btn btn-default pull-right'
            ]
        ];

        $helper = new HelperForm();

        // Module, token and currentIndex
        $helper->module = $this;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex.'&configure='.$this->name;

        // Language
        $helper->default_form_language = $defaultLang;
        $helper->allow_employee_form_lang = $defaultLang;

        // Title and toolbar
        $helper->title = $this->displayName;
        $helper->show_toolbar = false;
        $helper->toolbar_scroll = false;
        $helper->submit_action = 'submit'.$this->name;
        $helper->toolbar_btn = [
            'save' => [
                'desc' => $this->l('Save'),
                'href' => AdminController::$currentIndex.'&configure='.$this->name.'&save'.$this->name.
                    '&token='.Tools::getAdminTokenLite('AdminModules'),
            ],
            'back' => [
                'href' => AdminController::$currentIndex.'&token='.Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            ]
        ];

        // Load current value
        $helper->fields_value['UCC_PLUGIN_TOKEN'] = Tools::getValue('UCC_PLUGIN_TOKEN', Configuration::get('UCC_PLUGIN_TOKEN'));

        return $helper->generateForm($fieldsForm);
    }

    /**
     * @param $params
     * @return mixed
     */
    public function hookDisplayProductAdditionalInfo($params)
    {
        if (!$this->isConfigured()) {
            return null;
        }

        $this->context->smarty->assign('ucc_price', $params['product']['price_amount']);
        $this->context->smarty->assign('ucc_id', $params['product']['ean13']);
        $this->context->smarty->assign('ucc_base_id', $params['product']['id']);

        return $this->fetch('module:ucc/views/templates/hook/product.tpl');
    }

    /**
     * @param $params
     * @return mixed
     */
    public function hookDisplayOrderConfirmation($params)
    {
        if (!$this->isConfigured()) {
            return null;
        }

        $data = [];
        foreach ($params['order']->getProducts() as $product) {
            $data[] = [
                'id'    => $product['ean13'],
                'qty'   => $product['product_quantity'],
                'price' => $product['product_price_wt'],
                'baseId'=> $product['id']
            ];
        }

        $this->context->smarty->assign('ucc_products', $data);

        return $this->fetch('module:ucc/views/templates/hook/order-confirmation.tpl');
    }

    /**
     * @param $params
     * @return void
     */
    public function hookActionFrontControllerSetMedia($params)
    {
        if (!$this->isConfigured()) {
            return;
        }

        $this->context->controller->registerJavascript(
            'ucc-remote',
            'https://app.ucancommerce.com/assets/scripts/ucc-pixel.js',
            [
                'server'    => 'remote',
                'position'  => 'bottom',
                'priority'  => 20
            ]
        );

        $this->context->controller->registerJavascript(
            'ucc-bottom',
            'modules/ucc/views/js/ucc_bottom.js',
            [
                'position'  => 'bottom',
                'priority'  => 1000000
            ]
        );

        switch ($this->context->controller->php_self) {
            case 'product':
                $this->context->controller->registerJavascript(
                    'ucc-product',
                    'modules/ucc/views/js/ucc_product.js',
                    [
                        'position'  => 'bottom',
                        'priority'  => 1000010
                    ]
                );

                break;
            case 'order-confirmation':
                $this->context->controller->registerJavascript(
                    'ucc-order-confirmation',
                    'modules/ucc/views/js/ucc_order_confirmation.js',
                    [
                        'position'  => 'bottom',
                        'priority'  => 1000010
                    ]
                );

                break;
        }
    }
}
